#include "main.h"

Screen::Screen()
{
 nn_saved=0;

 hc=GetStdHandle(STD_OUTPUT_HANDLE);
 if (hc==INVALID_HANDLE_VALUE)
   {
    printf("GetStdHandle() failed (stdout), Error=%d\n",GetLastError());
    exit(255);
   }
 hk=GetStdHandle(STD_INPUT_HANDLE);
 if (hk==INVALID_HANDLE_VALUE)
   {
    printf("GetStdHandle() failed (stdin), Error=%d\n",GetLastError());
    exit(255);
   }

 CONSOLE_SCREEN_BUFFER_INFO ci;
 if (GetConsoleScreenBufferInfo(hc,&ci)==FALSE)
   {
    printf("GetConsoleScreenBufferInfo() failed, Error=%d\n",GetLastError());
	exit(255);
   }

 GetConsoleMode(hc,&def_console_mode);
 SetConsoleMode(hc,0);
 GetConsoleMode(hk,&def_keyboard_mode);

 nr=25; nc=80;
 crds.Y=nr; crds.X=nc; crdu.Y=crdu.X=0;	    // COORDs

 srect.Top=srect.Left=0;					// SMALL_RECT
 srect.Right=nc-1; srect.Bottom=nr-1;

 buf=new CHAR_INFO[nr*nc];
 ReadConsoleOutput(hc,buf,crds,crdu,&srect);

 cr=ci.dwCursorPosition.Y; 					// cursor row
 cc=ci.dwCursorPosition.X;					// cursor column

 fg=FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE;
 bg=0; in=0; 
 def_fg=fg; def_bg=bg; def_in=in;

 lprinter=(FILE *)0;		// local printer stream

 standout=0; local_echo=1;
}


Screen::~Screen()
{
 SetConsoleMode(hc,def_console_mode);
 SetConsoleMode(hk,def_keyboard_mode);
 delete buf;
}

int Screen::Process()
{
 unsigned i=0;
 if (tcp.GetBufferLen())
   {
    if (tcp.GetBuffer()[i]!=ESC)
      for (i=0; i<tcp.GetBufferLen() && tcp.GetBuffer()[i]!=ESC; i++) Putc(tcp.GetBuffer()[i]);
	else
	  {
	   int suc=PutEscape((BYTE *)tcp.GetBuffer(),tcp.GetBufferLen(),i);
	   if (suc) i=suc;
	  }
   }
 tcp.BufferDel(i);
 return i;
}

void Screen::Refresh()
{
 WriteConsoleOutput(hc,buf,crds,crdu,&srect);
 
 COORD coord; coord.Y=cr; coord.X=cc;	// set cursor position
 SetConsoleCursorPosition(hc,coord);
}


void Screen::Echo(unsigned char *s)
{
 if (local_echo)
   {
	int ln=strlen((char *)s);
    for (int i=0; i<ln; i++) Putc(s[i]);
	Refresh();
   }
}

void Screen::Echo(unsigned char b)
{
 if (local_echo) {Putc(b); Refresh();}
}


void Screen::Putc(unsigned char ch)
{
 int pos=cr;

 if (ch)
   {
    if (par.koi8) TransMem(&ch,1,koi2alt);
	if (lprinter) fprintf(lprinter,"%c",ch);
    else switch (ch)
		  {
		   case (unsigned char)BELL:	// BELL
		   		MessageBeep(0xFFFFFFFF);
		   		break;
		   case (unsigned char)'\t':	// Tab right
		   		cc=((cc+TABSIZE)/TABSIZE)*TABSIZE;
		   		if (cc>nc-1) {
					cc=0; cr++;
					if (cr>nr-1) {cr=nr-1; ScrollForward();}
				}
				break;
		   case (unsigned char)'\b':	// Backspace
		   		if (cc) cc--;
		   		break;
		   case (unsigned char)'\n':	// NL control char
		   		cr++; cc=0;
		   		if (cr>nr-1) {cr=nr-1; ScrollForward();}
				break;
		   case (unsigned char)'\r':	// CR control code
			    cc=0;
		   		break;
		   default:						// 'normal' ASCII characters
		        {
				 int pos=cr*nc+cc;
				 // check for operate in graphics mode
				 if (gmode) ch=ch^0x80;
				 buf[pos].Char.AsciiChar=ch;

				 // check for operate in standout mode
				 if (standout) buf[pos].Attributes=(bg>>4)|(fg<<4);
				 else buf[pos].Attributes=fg|bg|in;

				 cc++;
		   		 if (cc>nc-1)
		   		   {
		   		    cc=0; cr++;
				    if (cr>nr-1) {cr=nr-1; ScrollForward();}
		   		   }
				}
		  }
   }
}

int Screen::Puts(void *fmt,...)
{
 static char tmp[PUTS_BUFFER+1];
int i=0;
 va_list pvar;  va_start(pvar,fmt);
 vsprintf(tmp,(char *)fmt,pvar); va_end(pvar);
 for (i=0; tmp[i]; i++) Putc((unsigned char)tmp[i]);
 return i;
}

int Screen::PutAnsi(unsigned char *buf,int ln)
{
 int retv=0;
 const int nParam = 10;  // Maximum number of parameters
 int nparam[MAX_NPARAM];
 int np=0;
 int j=0;

 // processing numeric parameters
 for (j=0; j<ln && !ESC_END(buf[j]); j++)
    {
	 int ss=ESC_END(buf[j]);
	 if (isdigit(buf[j]))
	   {
	    int ss=!ESC_END(buf[j]);
		if (np<MAX_NPARAM) nparam[np++]=atoi((char *)&buf[j]);
		for (; j<ln && isdigit(buf[j]); j++) ;
		if (j<ln) j--;
	   }
	}

  if (j<ln)
   {
    retv=j+1;
    switch (buf[j])
          {
           // Set cursor position.
           case 'H':
		   case 'f':
       			if (np<2) nparam[1]=1;
       			if (np<1) nparam[0]=1;
            	SetCursorPosition(nparam[0]-1,nparam[1]-1);
            	break;
           // Set cursor row.
		   case 'd':
       			if (np<1) nparam[0]=cr;
            	SetCursorPosition(cr,nparam[0]-1);
            	break;
           // Move cursor up.
           case 'A':
       			if (np<1) nparam[0]=1;
            	MoveCursorPosition(-nparam[0],0);
            	break;
		   // Cursor to begin, n lines up
           case 'F':
     			if (np<1) nparam[0]=1;
            	MoveCursorPosition(-nparam[0],0);
				cc=0;
            	break;
           // Move cursor down.
           case 'B':
		   case 'e':
     			if (np<1) nparam[0]=1;
            	MoveCursorPosition(nparam[0],0);
            	break;
		   // Cursor to begin, n lines down
           case 'E':
     			if (np<1) nparam[0]=1;
            	MoveCursorPosition(nparam[0],0);
				cc=0;
            	break;
           // Move cursor right.
           case 'C':
		   case 'a':
            	if (np<1) nparam[0]=1;
            	MoveCursorPosition(0,nparam[0]);
            	break;
           // Move cursor left.
           case 'D':
 				if (np<1) nparam[0]=1;
        		MoveCursorPosition(0,-nparam[0]);
            	break;
           // Clear screen
           case 'J':
                {
           		 if (np<1) nparam[0]=0; gmode=0;
                 switch (nparam[0])	{
					 case 0: ClearEOScreen(); break;
                     case 1:	ClearBOScreen(); break;
                     case 2:	ClearScreen(); break;
				 }
                 break;
                }
           // Clear line
           case 'K':
           		{
                 if (np<1) nparam[0]=0;
                 switch (nparam[0]) {
					 case 0: ClearEOLine(); break;
                     case 1: ClearBOLine(); break;
                     case 2: ClearLine(); break;
				 }
                 break;
                }
           // Erase n characters
           case 'X':
           		{
                 if (np<1) nparam[0]=1;
				 int pos=cr*nc;
				 for (int i=0; i<nparam[0]; i++) {
					 Screen::buf[pos+i].Char.AsciiChar=' ';
				 }
				 break;
                }
	       // Color managing
           case 'm':
                {
				 if (np==1 && nparam[0]==0) {bg=0; ColorManage(0); ColorManage(37);}
				 else if (np<1) {np=1; nparam[0]=0;}
				 for (int i=0; i<np && i<3; i++)
					ColorManage(nparam[i]);
                 break;
                }
		   // Scroll forward
		   case 'S':
		        {
                 if (np<1) nparam[0]=1;
				 for (int i=0; i<nparam[0]; i++)
					ScrollForward();
				 break;
				}
		   // Scroll Backward
		   case 'T':
		        {
                 if (np<1) nparam[0]=1;
				 for (int i=0; i<nparam[0]; i++)
			         ScrollBackward();
				 break;
				}
		   // Delete Line
		   case 'M':
		        {
	             if (np<1) nparam[0]=1;
				 for (int i=0; i<nparam[0]; i++)
			         DeleteLine();
				 break;
				}
		   // Insert Line
		   case 'L':
		        {
                 if (np<1) nparam[0]=1;
				 for (int i=0; i<nparam[0]; i++)
			         InsertLine();
				 break;
				}
		   // Delete Character
		   case 'P':
		        {
	             if (np<1) nparam[0]=1;
				 for (int i=0; i<nparam[0]; i++)
			         DeleteChar();
				 break;
				}
		   // Insert character
		   case '@':
		        {
		         if (np<1) nparam[0]=1;
				 for (int i=0; i<nparam[0]; i++)
			 		 InsertChar();
				 break;
				}
		   // Local printing
           case 'i':
           		{
                 if (np<1) nparam[0]=4;
                 switch (nparam[0])
                       {
                        case 5: StartLocalPrinting(); break;
                        case 4: EndLocalPrinting(); break;
                        default: EndLocalPrinting(); break;
                       }
                 break;
                }
		   case 'Z':
			    {
					if (np<1) nparam[0]=1;
					for (int i=0; i<nparam[0]; i++) {
		   				cc=((cc-TABSIZE)/TABSIZE+1)*TABSIZE;
		   				if (cc<0) cc=0;
					}
					break;
				}
		   // ----------------------------------------------
		   // iBSCe2 mode
           case 'g':
           		{
                 break;
                }
		   // Save current cursor position
           case 's':
           		{
				 CursorSavePosition();
                 break;
                }
		   // Restore current cursor position
           case 'u':
           		{
				 CursorRestPosition();
                 break;
                }
		   // Switch to screen n
           case 'z':
           		{
                 break;
                }
		   // Default - unknown commands
		   default: ;
          }
   }
 return retv;
}

int Screen::PutAnsiExt(unsigned char *buf,int ln)
{
 int retv=0;
 const int nParam = 10;  // Maximum number of parameters
 int nparam[MAX_NPARAM];
 int np=0;
 int j=0;

 // processing numeric parameters
 for (j=0; j<ln && !ESC_END(buf[j]); j++)
    {
	 int ss=ESC_END(buf[j]);
	 if (isdigit(buf[j]))
	   {
	    int ss=!ESC_END(buf[j]);
		if (np<MAX_NPARAM) nparam[np++]=atoi((char *)&buf[j]);
		for (; j<ln && isdigit(buf[j]); j++) ;
		if (j<ln) j--;
	   }
	}

  if (j<ln)
   {
    retv=j+1;
    switch (buf[j])
          {
           // Set overscan color to c
           case 'A':	// Can't be implemented
            	break;
           // Set the bell parameter
		   case 'B':	// Can't be implemented
            	break;
           // Set set cursor size
           case 'C':
       			if (np<2) nparam[1]=0;
	           	SetCursorSize(nparam[0]-1,nparam[1]-1);
            	break;
           // Turn on/off intensity of BG color
           case 'D':
           case 'E':	// Blink mode Like 6845 CRT Controller
       			if (np<1) nparam[0]=0;
				if (nparam[0]) in=BACKGROUND_INTENSITY;
				else in&=((DWORD)-1)^BACKGROUND_INTENSITY;
            	break;
           // Set normal foreground to c
           case 'F':
			   {
				 if (np>0) SetExtendedColor(nparam[0],0);
				 break;
			   }
           // Alternative graphics mode
           case 'g':	// Can't be implemented
            	break;
           // Sets normal background
           case 'G':
			   {
				 if (np>0) SetExtendedColor(nparam[0],1);
				 break;
			   }
           // Set reversed background
           case 'I':	// Can't be implemented
            	break;
           // Set graphic foreground
           case 'J':	// Can't be implemented
            	break;
           // Set graphic background
           case 'K':	// Can't be implemented
            	break;
           // Set new regions fill mode
           case 'L':	// Can't be implemented
            	break;
           // Return current foreground color attributes
           case 'M':	// Can't be implemented
            	break;
		   // Default - unknown commands
		   default: ;
          }
   }
 return retv;
}

int Screen::PutEscape(unsigned char*buf,int ln,int beg)
{
 int retv=0;
 beg++;
 switch (buf[beg])
   	   {
		// dosansi
		case (unsigned char)'G':	// home key
			{
            	MoveCursorPosition(0,-cc);
            	retv=2; break;
			}
		case (unsigned char)'H':	// up
			{
            	MoveCursorPosition(-1,0);
            	retv=2; break;
			}
		case (unsigned char)'P':	// down
			{
            	MoveCursorPosition(1,0);
            	retv=2; break;
			}
		case (unsigned char)'K':	// left
			{
            	MoveCursorPosition(0,-1);
            	retv=2; break;
			}
		case (unsigned char)'M':	// right
			{
            	MoveCursorPosition(0,-1);
            	retv=2; break;
			}
		case (unsigned char)'I':	// PgUp
			{
            	retv=2; break;
			}
		case (unsigned char)'Q':	// PgDn
			{
            	retv=2; break;
			}
		case (unsigned char)'O':	// end key
			{
            	retv=2; break;
			}
		
		// -------------
		case (unsigned char)'7':	// Save current cursor position
        	{
				CursorSavePosition();
				retv=2; break;
            }
		case (unsigned char)'8':	// Restored saved cursor position
        	{
				CursorRestPosition();
				retv=2; break;
            }
        case (unsigned char)'[': 	// Extended SCO ANSI commands
        	{
				int i=0;
				if (buf[beg+1]!='=') {
        			// Check if we have whole escape sequence in buffer.
					for (i=beg; i<ln && !ESC_END(buf[i]); i++) ;
					if (i<ln) {
						int suc=PutAnsi(&buf[beg+1],ln-beg-1);
						if (suc) retv+=suc+2;
					}
					break;
				} else {
					for (i=beg; i<ln && !ESC_END(buf[i]); i++) ;
					if (i<ln) {
						int suc=PutAnsiExt(&buf[beg+2],ln-beg-1);
						if (suc) retv+=suc+3;
					}
				}
				break;
            }
		default: if (ln>1) {Puts("\x1b",1); retv+=1;}
       }
 return retv;
}


void Screen::ClearScreen()
{
 int pos=0;
 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;

 for (int r=0; r<nr; r++)
    {
     for (int c=0; c<nc; c++) buf[pos+c]=ci;
	 pos+=nc;
	}
 cr=cc=0;
}


void Screen::ClearEOScreen()
{
 int begcc=cc;
 int pos=cr*nc;
 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;

 for (int r=cr; r<nr; r++)
    {
     for (int c=begcc; c<nc; c++) buf[pos+c]=ci;
	 pos+=nc; begcc=0;
	}
}

void Screen::ClearBOScreen()
{
 int begcc=nc-1;
 int pos=0;
 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;

 for (int r=0; r<=cr; r++)
    {
	 if (r==cr) begcc=cc;
     for (int c=0; c<=begcc; c++) buf[pos+c]=ci;
	 pos+=nc;
	}
}

void Screen::ClearLine()
{
 int pos=cr*nc;
 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;

 for (int c=0; c<nc; c++) buf[pos+c]=ci;
}

void Screen::ClearEOLine()
{
 int pos=cr*nc;
 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;

 for (int c=cc; c<nc; c++) buf[pos+c]=ci;
}

void Screen::ClearBOLine()
{
 int pos=cr*nc;
 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;

 for (int c=0; c<cc; c++) buf[pos+c]=ci;
}

void Screen::SetCursorPosition(int r,int c)
{
 if (r<nr) cr=r; else cr=nr-1;
 if (c<nc) cc=c; else cc=nc-1;
}

int Screen::GetCursorRow() {return cr;}

int Screen::GetCursorCol() {return cc;}

void Screen::MoveCursorPosition(int r,int c) {SetCursorPosition(cr+r,cc+c);}

void Screen::ColorManage(int clr)
{
	switch (clr) {
	case 0:		in=0; standout=0; break;
	case 1:		in=FOREGROUND_INTENSITY; break;
	case 4:		break;
	case 5:		break;
	case 7:		standout=1; break;
	case 8:		fg=bg=0; in=0; standout=0; break;
	case 10:	gmode=0; in=0; break;
	case 11:	break;
	case 12:	gmode=1; break;
	case 30:	fg=0; break;
	case 31:	fg=FOREGROUND_RED; break;
	case 32:	fg=FOREGROUND_GREEN; break;
	case 33:	fg=FOREGROUND_RED|FOREGROUND_GREEN; break;
	case 34:	fg=FOREGROUND_BLUE; break;
	case 35:	fg=FOREGROUND_BLUE|FOREGROUND_RED; break;
	case 36:	fg=FOREGROUND_GREEN|FOREGROUND_BLUE; break;
	case 37:	fg=FOREGROUND_BLUE|FOREGROUND_GREEN|FOREGROUND_RED; break;
	case 38:	fg=FOREGROUND_BLUE|FOREGROUND_GREEN|FOREGROUND_RED; break;
	case 39:	break;
	case 40:	bg=0; break;
	case 41:	bg=BACKGROUND_RED; break;
	case 42:	bg=BACKGROUND_GREEN; break;
	case 43:	bg=BACKGROUND_RED|BACKGROUND_GREEN; break;
	case 44:	bg=BACKGROUND_BLUE; break;
	case 45:	bg=BACKGROUND_BLUE|BACKGROUND_RED; break;
	case 46:	bg=BACKGROUND_GREEN|BACKGROUND_BLUE; break;
	case 47:	bg=BACKGROUND_BLUE|BACKGROUND_GREEN|BACKGROUND_RED; break;
	default: ;
	}
}


void Screen::ScrollForward()
{
 for (int r=0; r<nr-1; r++)
    memcpy(&buf[r*nc],&buf[(r+1)*nc],nc*sizeof(CHAR_INFO));
 int pos=(nr-1)*nc;

 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=def_fg|def_bg|def_in;
 for (int c=0; c<nc; c++) buf[pos+c]=ci;
}

void Screen::ScrollBackward()
{
 for (int r=nr-1; r>0; r--)
    memcpy(&buf[r*nc],&buf[(r-1)*nc],nc*sizeof(CHAR_INFO));

 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=def_fg|def_bg|def_in;
 for (int c=0; c<nc; c++) buf[c]=ci;
}

void Screen::DeleteLine()
{
 for (int r=cr; r<nr-1; r++)
    memcpy(&buf[r*nc],&buf[(r+1)*nc],nc*sizeof(CHAR_INFO));
 int pos=(nr-1)*nc;

 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;
 for (int c=0; c<nc; c++) buf[pos+c]=ci;
}

void Screen::InsertLine()
{
 for (int r=nr-1; r>cr; r--)
    memcpy(&buf[r*nc],&buf[(r-1)*nc],nc*sizeof(CHAR_INFO));
 int pos=cr*nc;

 CHAR_INFO ci; ci.Char.AsciiChar=' '; ci.Attributes=fg|bg|in;
 for (int c=0; c<nc; c++) buf[pos+c]=ci;
}

void Screen::DeleteChar()
{
 int pos=cr*nc;
 for (int c=cc; c<nc-1; c++) buf[pos+c]=buf[pos+c+1];
 buf[pos+nc-1].Char.AsciiChar=' ';
}

void Screen::InsertChar()
{
 int pos=cr*nc;
 for (int c=nc-1; c>cc; c--) buf[pos+c]=buf[pos+c-1];
 buf[pos+cc].Char.AsciiChar=' ';
}

void Screen::StartLocalPrinting()
{
 lprinter=fopen("prn","w");
 if (!lprinter) MessageBox(0,"Can't attach your local printer device!","Error",MB_OK);
}

void Screen::EndLocalPrinting()
{
 if (lprinter)
   {
    fclose(lprinter);
	lprinter=(FILE *)0;
   }
}

void Screen::CursorSavePosition()
{
	if (nn_saved<MAX_SAVE) {
		cr_saved[nn_saved]=cr;
		cc_saved[nn_saved]=cc;
		nn_saved++;
	}
}

void Screen::CursorRestPosition()
{
	if (nn_saved>0) nn_saved--;
	else cr_saved[nn_saved]=cr_saved[nn_saved]=0;
	cr=cr_saved[nn_saved];
	cc=cc_saved[nn_saved];
}

void Screen::SetCursorSize(int s,int e)
{
	CONSOLE_CURSOR_INFO cci;
	cci.dwSize=(DWORD)(abs(s-e)/0.16);
	cci.bVisible=TRUE;
	SetConsoleCursorInfo(hc,&cci);
}

void Screen::SetExtendedColor(int clr,int bg_flag)
{
 int bgc=clr/16;
 int fgc=clr%16;

 if (bg_flag) {
	 int tmp=fgc; fgc=bgc; bgc=tmp;
 }

 if (!bg_flag) {

 switch (fgc) {
	 case 0: // black
		 fg=0; break;
	 case 1: // blue
		 fg=FOREGROUND_BLUE;
		 break;
	 case 2: // green
		 fg=FOREGROUND_GREEN;
		 break;
	 case 3: // cyan
		 fg=FOREGROUND_BLUE|FOREGROUND_GREEN;
		 break;
	 case 4: // red
		 fg=FOREGROUND_RED;
		 break;
	 case 5: // magneta
		 fg=FOREGROUND_BLUE|FOREGROUND_RED;
		 break;
	 case 6: // brown
		 fg=FOREGROUND_GREEN|FOREGROUND_RED;
		 break;
	 case 7: // white
		 fg=FOREGROUND_BLUE|FOREGROUND_GREEN|FOREGROUND_RED;
		 break;
	 case 8: // grey
		 fg=0; break;
	 case 9: // lt.blue
		 fg=FOREGROUND_BLUE; 
		 break;
	 case 10:// lt.green
		 fg=FOREGROUND_GREEN;
		 break;
	 case 11:// lt.cyan
		 fg=FOREGROUND_BLUE|FOREGROUND_GREEN;
		 break;
	 case 12:// lt.red
		 fg=FOREGROUND_RED;
		 break;
	 case 13:// lt.magneta
		 fg=FOREGROUND_BLUE|FOREGROUND_RED;
		 break;
	 case 14:// yellow
		 fg=FOREGROUND_GREEN|FOREGROUND_RED;
		 break;
	 case 15:// lt.white
		 fg=FOREGROUND_BLUE|FOREGROUND_GREEN|FOREGROUND_RED;
		 break;
	 default: ;
	 }

	 if (fgc>7) in|=FOREGROUND_INTENSITY;
	 else in&=((DWORD)-1)^FOREGROUND_INTENSITY;

 } else {

switch (bgc) {
	 case 0: // black
		 bg=0; break;
	 case 1: // blue
		 bg=BACKGROUND_BLUE;
		 break;
	 case 2: // green
		 bg=BACKGROUND_GREEN;
		 break;
	 case 3: // cyan
		 bg=BACKGROUND_BLUE|BACKGROUND_GREEN;
		 break;
	 case 4: // red
		 bg=BACKGROUND_RED;
		 break;
	 case 5: // magneta
		 bg=BACKGROUND_BLUE|BACKGROUND_RED;
		 break;
	 case 6: // brown
		 bg=BACKGROUND_GREEN|BACKGROUND_RED;
		 break;
	 case 7: // white
		 bg=BACKGROUND_BLUE|BACKGROUND_GREEN|BACKGROUND_RED;
		 break;
	 case 8: // grey
		 bg=0; break;
	 case 9: // lt.blue
		 bg=BACKGROUND_BLUE;
		 break;
	 case 10:// lt.green
		 bg=BACKGROUND_GREEN;
		 break;
	 case 11:// lt.cyan
		 bg=BACKGROUND_BLUE|BACKGROUND_GREEN;
		 break;
	 case 12:// lt.red
		 bg=BACKGROUND_RED;
		 break;
	 case 13:// lt.magneta
		 bg=BACKGROUND_BLUE|BACKGROUND_RED;
		 break;
	 case 14:// yellow
		 bg=BACKGROUND_GREEN|BACKGROUND_RED;
		 break;
	 case 15:// lt.white
		 bg=BACKGROUND_BLUE|BACKGROUND_GREEN|BACKGROUND_RED;
		 break;
	 default: ;
	 }

 if (bgc>7) in|=BACKGROUND_INTENSITY;
 else in&=((DWORD)-1)^BACKGROUND_INTENSITY;

 }

}