#if !defined(__CONSOLE_TELNET_SCREEN)
#define __CONSOLE_TELNET_SCREEN 1

#include <stdlib.h>
#include <stdio.h>
#include <windows.h>

#define TABSIZE		8			// TAB size

#define BELL		7			// BELL control code
#define ESC			27			// Escape control code

#define MAX_NPARAM	5			// Max number of numeric parameters
#define MAX_SPARAM	10			// Max string length of num.parameters

#define PUTS_BUFFER 255			// Max size of Puts() buffer

#define NPR_END		(x==';')					// Num.param.delimiter
#define ESC_END(x)	(isalpha(x) || x=='@')		// Esc delimiter

#define MAX_SAVE	20			// max number of coords save

// pseudo-graphics characters
#define G1	'?'			// upper right corner
#define G2	'Z'			// upper left corner
#define G3	'@'			// lower left corner
#define G4	'Y'			// lower right corner
#define G5	';'			// upper right corner (double)
#define G6	'I'			// upper left corner (double)
#define G7	'H'			// lower left corner (double)
#define G8	'<'			// lower right corner (double)
#define GC	'E'			// centre character
#define Gc	'N'			// centre character (double)
#define GD	'B'			// down-tick character
#define Gd	'K'			// down-tick character (double)
#define GH	'D'			// horizontal line character
#define Gh	'M'			// horizontal line character (double)
#define GU	'A'			// up-tick character
#define Gu	'J'			// up-tick character (double)
#define GV	'3'			// vertical line character
#define Gv	'1'			// vertical line character (double)
#define GL	'2'			// left-tick character
#define Gl	'L'			// left-tick character (double)
#define GR	'C'			// right-tick character
#define Gr	'4'			// right-tick character (double)

class Screen {
	public:
		HANDLE hc;							// handle to console output device
		HANDLE hk;							// handle to console input device
		int local_echo;						// local echo=1
	protected:
		WORD fg;							// foreground RGB color
		WORD bg;							// backgroung RGB color
		WORD in;							// color intensity
		WORD def_fg;						// default foreground RGB color
		WORD def_bg;						// default backgroung RGB color
		WORD def_in;						// default color intensity
		DWORD def_console_mode;				// default console mode
		DWORD def_keyboard_mode;			// default keyboard mode
		int standout;						// standout flag
		int gmode;							// graphics mode flag
		int nr;								// number of rows
		int nc;								// number of columns
		int cr;								// cursor row
		int cc;								// cursor column
		CHAR_INFO *buf;						// virtual video buffer
		COORD crdu;							// 0,0 coordinates
		COORD crds;							// screensize as COORD
		SMALL_RECT srect;					// screen size as SMALL_RECT
		FILE *lprinter;						// local printer stream
		int cr_saved[MAX_SAVE+1];			// saved cursor rows
		int cc_saved[MAX_SAVE+1];			// saved cursor cols
		int nn_saved;
	public:
		// constructor
		Screen();
		~Screen();
		// member finctions
		int  Process();
		void Refresh();
		void Echo(unsigned char *);
		void Echo(unsigned char);
		void Putc(unsigned char);
		int  Puts(void *,...);
		int  PutEscape(unsigned char *,int,int);
		int  PutAnsi(unsigned char *,int);
		int  PutAnsiExt(unsigned char *,int);
		//
		void ClearScreen();
		void ClearEOScreen();
	    void ClearBOScreen();
		void ClearLine();
		void ClearEOLine();
		void ClearBOLine();
		void SetCursorPosition(int,int);
		void MoveCursorPosition(int,int);
		int  GetCursorRow();
		int  GetCursorCol();
		void ColorManage(int);
		void ScrollForward();
		void ScrollBackward();
		void DeleteLine();
		void InsertLine();
		void DeleteChar();
		void InsertChar();
		void StartLocalPrinting();
		void EndLocalPrinting();
		void CursorSavePosition();
		void CursorRestPosition();
		void SetCursorSize(int,int);
		void SetExtendedColor(int,int=0);
};


#endif
