//////////////////////////////////////////////////////////////////////
// CTcpIp.h: Base class implementation for TCP/IP interface
// (C) DATAC 1999
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TCPIP_H__AADEAFB1_083D_11E0_B6B1_002035F3688A__INCLUDED_)
#define AFX_TCPIP_H__AADEAFB1_083D_11E0_B6B1_002035F3688A__INCLUDED_

#include <stdio.h>
#include <process.h>
#include <winsock.h>

#define TCPIP_BUFFER_INI		0x7FFF	// receive bufer size
#define TCPIP_MAX_STRING_LEN	0x800	// string length limit
#define TCPIP_MAX_HOSTNAME_LEN	255		// host mae limit

enum {
	TCPIP_NOERROR,			// no errors
	TCPIP_ERR_BUFLIMIT,		// attempt to increase buffer larger than limit
	TCPIP_ERR_BUFMEMALLOC,	// error reallocating bufer - not enought memory
	TCPIP_ERR_WSA_BAD_IP,	// error initialising WINSOCK
	TCPIP_ERR_INVALIDSOCK,	// invalid cocket in constructor
	TCPIP_ERR_THREAD		// can't create thread
};

//////////////////////////////////////////////////////////////////////
// class CTcpIp declaration
//////////////////////////////////////////////////////////////////////
class CTcpIp {
	// internal input and output queue
	class CTcpIpBuffer {
	public:
		CRITICAL_SECTION m_crs;
		BYTE *m_buf;
		UINT m_buflen;
		UINT m_buflenmax;
		UINT m_errcode;
		CTcpIp *m_pIp;
	public:
		CTcpIpBuffer();
		~CTcpIpBuffer();
		// member functions
		const BYTE * Buf() {return m_buf;}	// buffer accecc
		UINT Len() {return m_buflen;}			// data length
		BOOL Realloc(UINT nsize);			// buffer realloc
		UINT GetError() {return m_errcode;}	// get error code
		// data access functions
		BOOL Add(char c);					// add single character
		BOOL Add(LPCSTR s);					// add ASCIIZ string
		BOOL Add(void *m,UINT ln);			// add binary data
		BOOL Del(UINT ln);					// delete and drag tail
		BYTE & operator[](int i) {return m_buf[i];}
		void Purge();
	} m_InBuf;

	// statistics
	struct CTcpIpStatistics {
		DWORD m_bytes_in;
		DWORD m_bytes_out;
	} m_Stats;

	// socket-related data members
	HANDLE m_rx_event;				// read event
	SOCKET m_s;						// socket
	static WSADATA m_wdata;			// for WSA init
	UINT m_errcode;					// error code
	BYTE m_host[TCPIP_MAX_HOSTNAME_LEN+1];	// host to connect
	UINT m_port;					// port to connect 
	DWORD m_timeout;				// timeout, ms
	BOOL m_bConnected;				// 'connected' flag
	BOOL m_bThreadActive;			// thread activity flag

public:
	CTcpIp::CTcpIp();
	CTcpIp::CTcpIp(SOCKET cs);
	virtual CTcpIp::~CTcpIp();

	// socket-related connection functions
	BOOL Connect(LPCSTR host,UINT port);		// connect to host
	BOOL Connect(LPCSTR host,LPCSTR srvname);	// connect to host by protocol name
	BOOL Disconnect(UINT wait_ms=1000);			// disconnect
	BOOL SetTimeout(DWORD timeout);				// set timeout

protected:
	// socket-related internal I/O functions (DON'T USE!)
	// (all members won't wait for any data!!!)
	BOOL __Recv(BYTE *b);
	BOOL __Recv(BYTE *b,UINT ln);
	UINT __GetSysBuffered();
	friend UINT CTcpIpIOThread(CTcpIp &ip);

public:
	// socket-related public I/O functions
	BOOL Send(BYTE b);				// send byte
	BOOL Send(void *mem,UINT ln);	// send binary block
	BOOL Send(LPCSTR str);			// send ASCIIZ string
	BOOL Sendf(LPCSTR fmt,...);		// send formatted string
	BYTE Getc(BOOL wait=TRUE,BOOL *err=NULL);	// get byte
	BOOL Recv(void *m,UINT ln, BOOL wait=TRUE);
	BOOL Gets(void *mem,UINT ln=TCPIP_MAX_STRING_LEN,BOOL wait=TRUE);

	// socket-related information function
	UINT GetError() {return m_errcode;}
	UINT IsConnected() {return m_bConnected;}

	// overridable members
	virtual void OnDisconnect();							// disconnect handler
	virtual void OnError(DWORD err,LPCSTR fname,UINT line); // error handler
	virtual BOOL OnRecvData(BYTE *buf,UINT ln);				// add to buf if return TRUE

	// socket-related public member functions for IP-server
	BOOL Bind(UINT port);		// by port
	BOOL Bind(LPCSTR srvname);	// by service name
	SOCKET Accept();

	// special buffer-access member function
	const BYTE * GetBuffer() {return m_InBuf.Buf();}
	UINT GetBufferLen() {return m_InBuf.Len();}
	BOOL BufferDel(UINT ln) {return m_InBuf.Del(ln);}

	// statistics functions
	DWORD GetStatInBytes() {return m_Stats.m_bytes_in;}
	DWORD GetStatOutBytes() {return m_Stats.m_bytes_out;}
	void ClearStatistics() {m_Stats.m_bytes_in=m_Stats.m_bytes_out=0;}
};

#endif
