//////////////////////////////////////////////////////////////////////
// CTcpIp.h: Base class implementation for TCP/IP interface
// (C) DATAC 1999
//////////////////////////////////////////////////////////////////////

#include "TcpIp.h"

#pragma optimize("gt",on)

///////////////////////////////////////////////////
// CTcpIpBuffer implementation

 
// CTcpIpBuffer constructor
CTcpIp::CTcpIpBuffer::CTcpIpBuffer()
{
	m_buflen=0;
	m_errcode=TCPIP_NOERROR;
	m_buflenmax=TCPIP_BUFFER_INI;
	m_pIp=NULL;
	InitializeCriticalSection(&m_crs);
	m_buf=new BYTE[m_buflenmax+1];
}

// CTcpIpBuffer denstructor
CTcpIp::CTcpIpBuffer::~CTcpIpBuffer()
{
	delete m_buf;
}

// Buffer realocation
BOOL CTcpIp::CTcpIpBuffer::Realloc(UINT nsize)
{
	LPCSTR fname="Realloc(UINT)";
	BOOL retv=TRUE;

	EnterCriticalSection(&m_crs);
	BYTE *tmp=(BYTE *)realloc(m_buf,nsize);
	if (tmp) {
		m_buf=tmp;
		m_buflenmax=nsize;
	} else {
		m_errcode=TCPIP_ERR_BUFMEMALLOC;
		m_pIp->OnError(m_errcode,fname,__LINE__);
		retv=FALSE;
	}

	LeaveCriticalSection(&m_crs);
	return retv;
}

// Add byte to buffer
BOOL CTcpIp::CTcpIpBuffer::Add(char b)
{
	// add byte to buffer
	EnterCriticalSection(&m_crs);
	m_buf[m_buflen++]=b;
	LeaveCriticalSection(&m_crs);

	return TRUE;
}

// Add string to buffer
BOOL CTcpIp::CTcpIpBuffer::Add(LPCSTR s)
{
	return Add((void *)s,strlen(s));
}

// Add binary data to buffer
BOOL CTcpIp::CTcpIpBuffer::Add(void *m, UINT ln)
{
	// add binary data to buffer
	EnterCriticalSection(&m_crs);
	memcpy(&m_buf[m_buflen],m,ln);
	m_buflen+=ln;
	LeaveCriticalSection(&m_crs);

	return TRUE;
}

// delete n bytes from buffer
BOOL CTcpIp::CTcpIpBuffer::Del(UINT n)
{
	if (n>m_buflen) n=m_buflen;
	EnterCriticalSection(&m_crs);
	for (UINT i=0; i<m_buflen-n; i++) m_buf[i]=m_buf[i+n];
	m_buflen-=n; m_buf[m_buflen]=0;
	LeaveCriticalSection(&m_crs);
	return TRUE;
}

// Purge any data from buffer ans realloc memory
void CTcpIp::CTcpIpBuffer::Purge()
{
	EnterCriticalSection(&m_crs);
	m_buflen=0;
	LeaveCriticalSection(&m_crs);
	Realloc(TCPIP_BUFFER_INI);
}

///////////////////////////////////////////////////
// CTcpIp I/O thread
static UINT CTcpIpIOThread(CTcpIp &ip)
{
	LPCSTR fname="CTcpIpIOThread()";

	BYTE *tmp;
	UINT tmpl,lnr;

	// clear statustics
	ip.ClearStatistics();

	// temporary buffer
	ip.m_bThreadActive=TRUE;
	tmpl=TCPIP_BUFFER_INI;
	tmp=new BYTE[tmpl+1];


	fd_set fd_rd; FD_ZERO(&fd_rd); FD_SET(ip.m_s,&fd_rd);

	ip.m_rx_event=CreateEvent(NULL,FALSE,FALSE,NULL);

	while  (ip.IsConnected()) {

		// waiting for income data or disconnect
		if (select(0,&fd_rd,NULL,NULL,NULL)==SOCKET_ERROR) {
			ip.OnError(WSAGetLastError(),fname,__LINE__);
			break;
		}

		// checking for read data
		if (FD_ISSET(ip.m_s,&fd_rd)) {
			lnr=ip.__GetSysBuffered();

			if (lnr<1) {
				ip.m_bConnected=FALSE;
				ip.OnDisconnect();
				break;
			} else ip.m_bConnected=TRUE;

			// update sataistics
			ip.m_Stats.m_bytes_in+=lnr;

			// check if buffer is full
			if (lnr+ip.m_InBuf.m_buflen>=ip.m_InBuf.m_buflenmax) {
				lnr=ip.m_InBuf.m_buflenmax-ip.m_InBuf.m_buflen;
				Sleep(200);
			}

			if (lnr>tmpl) {
				tmpl=lnr;
				tmp=(BYTE *)realloc(tmp,tmpl+1);
			}

			if (ip.__Recv(tmp,lnr)) {
				if (ip.OnRecvData(tmp,lnr)) {
					if (!ip.m_InBuf.Add(tmp,lnr)) {
						ip.OnError(ip.m_InBuf.GetError(),fname,__LINE__);
						break;
					}
					SetEvent(ip.m_rx_event);
				}

			}
		}

	}

	closesocket(ip.m_s);
	delete tmp;

	ip.m_bThreadActive=FALSE;

	CloseHandle(ip.m_rx_event);
	ip.m_rx_event=NULL;

	ExitThread(0);
	return 0;
}


///////////////////////////////////////////////////
// CTcpIp implementation

// static member
WSADATA CTcpIp::m_wdata;		// for WSA init

// CTcpIp constructor
CTcpIp::CTcpIp()
{
	LPCSTR fname="CTcpIp() constuctor";

	m_errcode=TCPIP_NOERROR;
	m_s=INVALID_SOCKET;
	m_bConnected=FALSE;
	m_timeout=0;
	m_InBuf.m_pIp=this;
	m_bThreadActive=FALSE;
	m_rx_event=NULL;
	ClearStatistics();

	if (WSAStartup(MAKEWORD(1,1),&m_wdata)) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
	}
}

// CTcpIp destructor
CTcpIp::~CTcpIp()
{
	 Disconnect();
}

// CTcpIp constructor for server usage
CTcpIp::CTcpIp(SOCKET cs)
{
	LPCSTR fname="CTcpIp(SOCKET) constuctor";

	m_errcode=TCPIP_NOERROR;
	m_InBuf.m_pIp=this;
	m_timeout=0;
	m_s=cs;
	m_bThreadActive=FALSE;
	m_rx_event=NULL;
	ClearStatistics();

	if (m_s==INVALID_SOCKET) {
		m_errcode=TCPIP_ERR_INVALIDSOCK;
		m_bConnected=FALSE;
	} else {
		if (WSAStartup(MAKEWORD(1, 1),&m_wdata)) {
			m_errcode=WSAGetLastError();
			m_bConnected=FALSE;
			OnError(m_errcode,fname,__LINE__);
		} else {
			// Create I/O thread
			BOOL b_flag=TRUE;

			m_bConnected=TRUE;
			m_InBuf.Purge();		// clear buffer

			if (setsockopt(m_s,IPPROTO_TCP,SO_KEEPALIVE,(LPCSTR)&b_flag,sizeof(BOOL))!=NO_ERROR) {
				m_errcode=WSAGetLastError();
				OnError(m_errcode,fname,__LINE__);
			} else {
				DWORD id;
				HANDLE htr=CreateThread(0,0,(LPTHREAD_START_ROUTINE)CTcpIpIOThread,
					(LPVOID)this,THREAD_SET_INFORMATION,&id);
				if (htr==NULL) {
					m_errcode=TCPIP_ERR_THREAD;
					OnError(m_errcode,fname,__LINE__);
				} else {
					SetThreadPriority(htr,THREAD_PRIORITY_ABOVE_NORMAL);
				}
			}
		}
	}
}

// Internal Recv function
BOOL CTcpIp::__Recv(BYTE *b) {

	LPCSTR fname="__Recv(BYTE *)";

	m_errcode=NO_ERROR;
	if (recv(m_s,(char *)b,1,0)==1) return TRUE;
	else {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
}

// Internal Recv function
BOOL CTcpIp::__Recv(BYTE *b,UINT len) {

	LPCSTR fname="__Recv(BYTE *,UINT)";
	
	m_errcode=NO_ERROR;
retry:
	int suc=recv(m_s,(char *)b,len,0);
	if (suc!=SOCKET_ERROR) {
		if (len==(UINT)suc) return TRUE;
		len=len-suc; b+=suc;
		goto retry;
	} else {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
	return FALSE;
}

// Get number of recieved bytes in system buffer
UINT CTcpIp::__GetSysBuffered() {

	LPCSTR fname="__GetSysBuffered(BYTE *,UINT)";

	DWORD ln=0;
	// check data aviable for reading
	if (ioctlsocket(m_s,FIONREAD,&ln)==NO_ERROR) 
		return ln;
	else {
		m_errcode=WSAGetLastError();
		if (m_errcode!=WSAENOTSOCK) OnError(m_errcode,fname,__LINE__);
		return 0;
	}
}

// Connect to server
BOOL CTcpIp::Connect(LPCSTR host,UINT port)
{

	LPCSTR fname="Connect(LPCSTR,UINT)";

	sockaddr_in sin;
	memset(&sin,0,sizeof(sin));

	// store valus for reconnect in future
	strncpy((char *)m_host,host,TCPIP_MAX_HOSTNAME_LEN);
	m_host[TCPIP_MAX_HOSTNAME_LEN]=0;
	m_port=port;

	sin.sin_family=AF_INET;
	sin.sin_port=htons(port);

	m_errcode=NO_ERROR;

	// Resolve host address if needed
	if (isdigit(host[0]) && inet_addr(host)!=INADDR_NONE) {
		sin.sin_addr.s_addr=inet_addr(host);
	} else {
		hostent *hp=gethostbyname((const char *)host);
		if (!hp) {
			m_errcode=WSAGetLastError();
			OnError(m_errcode,fname,__LINE__);
			return FALSE;
		}
		// storing address information
		memcpy(&sin.sin_addr,hp->h_addr,hp->h_length);
	}

	// Creating socket
	if ((m_s=socket(AF_INET,SOCK_STREAM,0))==INVALID_SOCKET) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}

	// Connect to host by IP-address
	if (connect(m_s,(sockaddr *)&sin,sizeof(sin))) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	} else {
		// Create I/O thread
		BOOL b_flag=TRUE;

		m_bConnected=TRUE;
		m_InBuf.Purge();		// clear buffer

		if (setsockopt(m_s,IPPROTO_TCP,SO_KEEPALIVE,(LPCSTR)&b_flag,sizeof(BOOL))!=NO_ERROR) {
			m_errcode=WSAGetLastError();
			OnError(m_errcode,fname,__LINE__);
			return FALSE;
		}

		DWORD id;
		HANDLE htr=CreateThread(0,0,(LPTHREAD_START_ROUTINE)CTcpIpIOThread,
			(LPVOID)this,THREAD_SET_INFORMATION,&id);
		if (htr==NULL) {
			m_errcode=TCPIP_ERR_THREAD;
			OnError(m_errcode,fname,__LINE__);
		} else {
			SetThreadPriority(htr,THREAD_PRIORITY_ABOVE_NORMAL);
		}
	}

	return TRUE;
}

// Connect to server by protocol name
BOOL CTcpIp::Connect(LPCSTR host,LPCSTR srvname)
{

	LPCSTR fname="Conect(LPCSTR,LPCSTR)";

	m_errcode=TCPIP_NOERROR;
	servent *srvinfo=getservbyname(srvname,NULL);
	if (srvinfo) {
		return Connect(host,srvinfo->s_port);
	} else {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
}

// Disconnect from server
BOOL CTcpIp::Disconnect(UINT wait_ms)
{
	LPCSTR fname="Disconnect(UINT)";

	m_errcode=TCPIP_NOERROR;

	if (!closesocket(m_s)) {
		m_errcode=WSAGetLastError();
		if (m_errcode) OnError(m_errcode,fname,__LINE__);
	}

	// wait for thread termination
	m_s=INVALID_SOCKET;
	if (wait_ms>0) {
		for (UINT wt=0; wt<wait_ms && m_bThreadActive; wt++)
			Sleep(1);
	} else {
		while (m_bThreadActive)	Sleep(1);
	}

	return TRUE;
}

BOOL CTcpIp::SetTimeout(DWORD timeout)
{
	LPCSTR fname="SetTimeout(UINT)";

	m_timeout=timeout;

	if (setsockopt(m_s,SOL_SOCKET,SO_RCVTIMEO,(char *)&timeout,sizeof(timeout))!=NO_ERROR) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
	if (setsockopt(m_s,SOL_SOCKET,SO_SNDTIMEO,(char *)&timeout,sizeof(timeout))!=NO_ERROR) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}

	return TRUE;
}

// if disconnect occured
void CTcpIp::OnDisconnect()
{
	Beep(5000,100);
	printf("\nConnection closed\n");
}

// if disconnect occured
void CTcpIp::OnError(DWORD err,LPCSTR fname,UINT line)
{
	Beep(3000,100);
	printf("\nError: %d in function '%s' Line %d  Module '%s'\n",err,fname,line,__FILE__);
}

// if some data recieved
BOOL CTcpIp::OnRecvData(BYTE *buf,UINT ln)
{
	// if you need to add data to buffer - return TRUE
	// if you don't need - return TRUE
	return TRUE;
}


// Send byte function
BOOL CTcpIp::Send(BYTE b)
{
	LPCSTR fname="Send(BYTE)";

	if (send(m_s,(LPCSTR)&b,sizeof(BYTE),0)==SOCKET_ERROR) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
	m_Stats.m_bytes_out++;

	return TRUE;
}

// Send binary data function
BOOL CTcpIp::Send(void *m,UINT ln)
{
	LPCSTR fname="Send(BYTE *,UINT)";
	BYTE *b=(BYTE *)m;

	int suc=send(m_s,(LPCSTR)b,ln,0);
retry:
	if (suc!=SOCKET_ERROR) {
		if (ln==(UINT)suc) return TRUE;
		ln=ln-suc; b+=suc;
		goto retry;
	} else {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
	m_Stats.m_bytes_out+=ln;

	return TRUE;
}

// Send string function
BOOL CTcpIp::Send(LPCSTR str)
{
	return Send((void *)str,strlen(str));
}

// Send formatted string function
BOOL CTcpIp::Sendf(LPCSTR fmt,...)
{
	char tmp[TCPIP_MAX_STRING_LEN+1];

	va_list pvar;  va_start(pvar,fmt);
	vsprintf(tmp,(char *)fmt,pvar); va_end(pvar);
	return Send(tmp);
}

// Receive data with (or withou waiting)
BOOL CTcpIp::Recv(void *m,UINT ln, BOOL wait)
{
	if (!wait) {
		// do not wait for data
		if (m_InBuf.Len()>=ln) {
			memcpy(m,m_InBuf.Buf(),ln);
			m_InBuf.Del(ln);
			return TRUE;
		} else return FALSE;
	} else {
		// wait for data
		if (m_timeout==0) {
			while (m_InBuf.Len()<ln) {
				if (!IsConnected()) return FALSE;
				WaitForSingleObject(m_rx_event,INFINITE);
			}
		} else {
			for (UINT t=0; t<m_timeout && m_InBuf.Len()<ln; t++) {
				if (!IsConnected()) return FALSE;
				Sleep(1);
			}
			if (m_InBuf.Len()<ln) return FALSE;
		}
		memcpy(m,m_InBuf.Buf(),ln);
		m_InBuf.Del(ln);
		return TRUE;
	}
	return TRUE;
}

// bind socket by specified port
BOOL CTcpIp::Bind(UINT port)
{
	LPCSTR fname="Bind(UINT)";

	int suc;
	m_s=socket(AF_INET,SOCK_STREAM,0);
	if (m_s==INVALID_SOCKET) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}

	sockaddr_in sin;
	sin.sin_family=AF_INET;
	sin.sin_port=htons(port);
	sin.sin_addr.S_un.S_addr=INADDR_ANY;
	int fromlen=sizeof(sin);

	int one=1;
	setsockopt(m_s,SOL_SOCKET,SO_REUSEADDR,(char *)&one, sizeof(one));

	suc=bind(m_s,(sockaddr *)&sin,fromlen);
	if (suc!=0) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
	return TRUE;
}

// bind socket by specified service name
BOOL CTcpIp::Bind(LPCSTR srvname)
{
	LPCSTR fname="Bind(LPCSTR)";

	servent *srvinfo=getservbyname(srvname,NULL);

	if (srvinfo) {
		return Bind(srvinfo->s_port);
	} else {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return FALSE;
	}
}

// listening and accepting connection
SOCKET CTcpIp::Accept()
{
	LPCSTR fname="Accept()";

	if (listen(m_s,5)) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return INVALID_SOCKET;
	}

	SOCKET ss=accept(m_s,NULL,NULL);

	if (ss==INVALID_SOCKET) {
		m_errcode=WSAGetLastError();
		OnError(m_errcode,fname,__LINE__);
		return INVALID_SOCKET;
	}

	return ss;
}

// get one byte and remove it from buffer
BYTE CTcpIp::Getc(BOOL wait,BOOL *err)
{
	BYTE retv=0;

retry:
	if (m_InBuf.Len()>0) {
		retv=m_InBuf[0];
		m_InBuf.Del(1);
		if (err) *err=FALSE;
	} else {
		if (wait) {
			WaitForSingleObject(m_rx_event,m_timeout);
			goto retry;
		} else {
			if (err) *err=TRUE;
		}
	}

	return retv;
}


// get string (\n) and remove it from buffer
BOOL CTcpIp::Gets(void *mem,UINT ln,BOOL wait)
{
	BOOL retv=FALSE;
	BYTE *b=(BYTE *)mem;
	UINT i;

	b[0]=0;

	if (wait) {
		if (m_timeout>0) {
			for (i=0; i<m_timeout; i++) {
				if (memchr((const void *)m_InBuf.Buf(),'\n',m_InBuf.Len())) break;
				else Sleep(1);
			}
		} else {
			while (!memchr((const void *)m_InBuf.Buf(),'\n',m_InBuf.Len()))
				Sleep(1);
		}
	}

	for (i=0; i<m_InBuf.Len(); i++) {
		if (m_InBuf[i]=='\r') continue;
		if (m_InBuf[i]=='\n') {
			retv=TRUE;
			UINT strl=i+1;
			if (strl>ln) strl=ln;
			memcpy(mem,m_InBuf.Buf(),strl);
			m_InBuf.Del(strl);
			b[strl]=0;
			break;
		}
	}

	return retv;
}


#pragma optimize( "", off )
